import hmac
import hashlib
import sys


from ..errors import SignatureVerificationError


class Utility(object):
    def __init__(self, client=None):
        self.client = client

    def verify_payment_signature(self, parameters):
        order_id = str(parameters['thawani_order_id'])
        payment_id = str(parameters['thawani_payment_id'])
        thawani_signature = str(parameters['thawani_signature'])
       
        msg = "{}|{}".format(order_id, payment_id)
        
        secret = str(self.client.auth[1])

        return self.verify_signature(msg, thawani_signature, secret)

    def verify_payment_link_signature(self, parameters):
        
        if 'thawani_payment_id' in parameters.keys() and 'payment_link_reference_id' in parameters.keys() and 'payment_link_status' in parameters.keys():
            payment_id = str(parameters['thawani_payment_id'])
            payment_link_id = str(parameters['payment_link_id'])
            payment_link_reference_id = str(parameters['payment_link_reference_id'])
            payment_link_status = str(parameters['payment_link_status'])
            thawani_signature = str(parameters['thawani_signature'])
        else:
            return False
          
        msg = "{}|{}|{}|{}".format(payment_link_id, payment_link_reference_id, payment_link_status, payment_id)
        
        secret = str(parameters['secret']) if 'secret' in parameters.keys() else str(self.client.auth[1])

        return self.verify_signature(msg, thawani_signature, secret)    
    
    def verify_subscription_payment_signature(self, parameters):
        """
        To consider the payment as successful and subscription as authorized 
        after the signature has been successfully verified
        """
        subscription_id = str(parameters['thawani_subscription_id'])
        payment_id = str(parameters['thawani_payment_id'])
        thawani_signature = str(parameters['thawani_signature'])

        msg = "{}|{}".format(payment_id, subscription_id)

        secret = str(parameters['secret']) if 'secret' in parameters.keys() else str(self.client.auth[1])

        return self.verify_signature(msg, thawani_signature, secret)
    
    def verify_webhook_signature(self, body, signature, secret):
        return self.verify_signature(body, signature, secret)

    def verify_signature(self, body, signature, key):
        if sys.version_info[0] == 3:  # pragma: no cover
            key = bytes(key, 'utf-8')
            body = bytes(body, 'utf-8')

        dig = hmac.new(key=key,
                       msg=body,
                       digestmod=hashlib.sha256)

        generated_signature = dig.hexdigest()

        if sys.version_info[0:3] < (2, 7, 7):
            result = self.compare_string(generated_signature, signature)
        else:
            result = hmac.compare_digest(generated_signature, signature)

        if not result:
            raise SignatureVerificationError(
                'thawani Signature Verification Failed')
        return result

    # Taken from Django Source Code
    # Used in python version < 2.7.7
    # As hmac.compare_digest is not present in prev versions
    def compare_string(self, expected_str, actual_str):
        """
        Returns True if the two strings are equal, False otherwise
        The time taken is independent of the number of characters that match
        For the sake of simplicity, this function executes in constant time only
        when the two strings have the same length. It short-circuits when they
        have different lengths
        """
        if len(expected_str) != len(actual_str):
            return False
        result = 0
        for x, y in zip(expected_str, actual_str):
            result |= ord(x) ^ ord(y)
        return result == 0



from .base import Resource
from ..constants.url import URL


class FundAccount(Resource):
    def __init__(self, client=None):
        super(FundAccount, self).__init__(client)
        self.base_url = URL.FUND_ACCOUNT_URL

    def all(self, data={}, **kwargs):
        """"
        Fetch all Fund Account entities

        Returns:
            Dictionary of Fund Account
        """
        return super(FundAccount, self).all(data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create a fund account

        Args:
            data : Dictionary having keys using which order have to be created
                'customerId' :  Customer Id for the customer
                'account_type' : The bank_account to be linked to the customer ID
                'bank_account' : key value pair

        Returns:
            fund account Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)


from .base import Resource
from ..constants.url import URL


class Plan(Resource):
    def __init__(self, client=None):
        super(Plan, self).__init__(client)
        self.base_url = URL.PLAN_URL

    def create(self, data={}, **kwargs):
        """"
        Create Plan from given dict

        Args:
            data : Dictionary having keys using which Plan has to be created

        Returns:
            Plan Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def fetch(self, plan_id, data={}, **kwargs):
        """"
        Fetch Plan for given Id

        Args:
            plan_id : Id for which Plan object has to be retrieved

        Returns:
            Plan dict for given subscription Id
        """
        return super(Plan, self).fetch(plan_id, data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch all plan entities

        Returns:
            Dictionary of plan data
        """
        return super(Plan, self).all(data, **kwargs)



from .base import Resource
from ..constants.url import URL
import warnings


class Refund(Resource):
    def __init__(self, client=None):
        super(Refund, self).__init__(client)
        self.base_url = URL.REFUNDS_URL

    def fetch_all(self, data={}, **kwargs):  # pragma: no cover
        warnings.warn("Will be Deprecated in next release, use all",
                      DeprecationWarning)
        return self.all(data, **kwargs)

    def create(self, data={}, **kwargs):
        """
        Create refund for given payment id
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch All Refund

        Returns:
            Refund dict
        """
        return super(Refund, self).all(data, **kwargs)

    def fetch(self, refund_id, data={}, **kwargs):
        """"
        Refund object for given paymnet Id

        Args:
            refund_id : Refund Id for which refund has to be retrieved

        Returns:
            Refund dict for given refund Id
        """
        return super(Refund, self).fetch(refund_id, data, **kwargs)

    def edit(self, refund_id, data={}, **kwargs):
        """"
        Update Refund

        Returns:
            Refund Dict which was edited
        """
        url = "{}/{}".format(self.base_url, refund_id)
        return self.patch_url(url, data, **kwargs)     



from .base import Resource
from ..constants.url import URL


class Qrcode(Resource):
    def __init__(self, client=None):
        super(Qrcode, self).__init__(client)
        self.base_url = URL.QRCODE_URL

    def fetch(self, qrcode_id, data={}, **kwargs):
        """"
        Fetch a Qr code

        Args:
            customer_id : Id for which customer object has to be retrieved

        Returns:
            Qrcode dict for given qrcode id
        """
        return super(Qrcode, self).fetch(qrcode_id, data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create a QR Code

        Returns:
            QrCode Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)
    
    def all(self, data={}, **kwargs):
        """"
        Fetch All Qr Code

        Returns:
            Qrcode dict
        """
        return super(Qrcode, self).all(data, **kwargs)

    def fetch_all_payments(self, qrcode_id,  data={}, **kwargs):
        """"
        Fetch Payments for a QR Code

        Returns:
            Qrcode payment dict
        """
        url = "{}/{}/payments".format(self.base_url, qrcode_id)
        return self.get_url(url, data, **kwargs)   

    def close(self, qrcode_id, **kwargs):
        """"
        Close a QR Code

        Returns:
            Qrcode Dict which was closed
        """
        url = '{}/{}/close'.format(self.base_url, qrcode_id)

        return self.post_url(url, {}, **kwargs)



import os
import json
import requests
import pkg_resources

from pkg_resources import DistributionNotFound

from types import ModuleType

from .constants import HTTP_STATUS_CODE, ERROR_CODE, URL

from . import resources, utility

from .errors import (BadRequestError, GatewayError,
                     ServerError)


def capitalize_camel_case(string):
    return "".join(map(str.capitalize, string.split('_')))


# Create a dict of resource classes
RESOURCE_CLASSES = {}
for name, module in resources.__dict__.items():
    if isinstance(module, ModuleType) and \
            capitalize_camel_case(name) in module.__dict__:
        RESOURCE_CLASSES[name] = module.__dict__[capitalize_camel_case(name)]

UTILITY_CLASSES = {}
for name, module in utility.__dict__.items():
    if isinstance(module, ModuleType) and name.capitalize() in module.__dict__:
        UTILITY_CLASSES[name] = module.__dict__[name.capitalize()]


class Client:
    """thawani client class"""

    def __init__(self, session=None, data=None, **options):
        """
        Initialize a Client object with session,
        optional auth handler, and options
        """
        self.session = session or requests.Session()
        self.secret_key =data.get('secret_key')
        self.publishable_key=data.get('publishable_key')        
        if data.get('use_sandbox')==1:
            self.base_url = URL.SANDBOX_URL  
            self.endpoint = URL.SANDBOX_ENDPOINT
        else:
            self.base_url = URL.PROD_URL  
            self.endpoint = URL.PROD_ENDPOINT


        print( self.secret_key)
        print( self.publishable_key)
        print( self.base_url)
        self.app_details = []
        # intializes each resource
        # injecting this client object into the constructor
        for name, Klass in RESOURCE_CLASSES.items():
            setattr(self, name, Klass(self))

        for name, Klass in UTILITY_CLASSES.items():
            setattr(self, name, Klass(self))


    def _update_user_agent_header(self, options):
        user_agent = "{}{} {}".format('thawani-Python/', self._get_version(),
                                      self._get_app_details_ua())

        if 'headers' in options:
            options['headers']['User-Agent'] = user_agent
        else:
            options['headers'] = {'User-Agent': user_agent}

        return options

    def _get_version(self):
        version = ""
        try: # nosemgrep : gitlab.bandit.B110
            version = pkg_resources.require("thawani")[0].version
        except DistributionNotFound:  # pragma: no cover
            pass
        return version

    def _get_app_details_ua(self):
        app_details_ua = ""

        app_details = self.get_app_details()

        for app in app_details:
            if 'title' in app:
                app_ua = app['title']
                if 'version' in app:
                    app_ua += "/{}".format(app['version'])
                app_details_ua += "{} ".format(app_ua)

        return app_details_ua

    def set_app_details(self, app_details):
        self.app_details.append(app_details)

    def get_app_details(self):
        return self.app_details

    def request(self, method, path, **options):
        """
        Dispatches a request to the thawani HTTP API
        """
        options = self._update_user_agent_header(options)
        url = "{}{}".format(self.base_url, path)
        if 'headers' in options:
            options['headers']['thawani-api-key'] = self.secret_key
        else:
            options['headers'] = {'thawani-api-key': self.secret_key}
        
        response = getattr(self.session, method)(url,**options)
        if ((response.status_code >= HTTP_STATUS_CODE.OK) and
                (response.status_code < HTTP_STATUS_CODE.REDIRECT)):
            return json.dumps({}) if(response.status_code==204) else response.json()
        else:
            msg = ""
            code = ""
            json_response = response.json()

            if json_response['success']==False:
                if 'description' in json_response:
                    msg = json_response['description']
                if 'code' in json_response:
                    code = str(json_response['code'])
            if str.upper(code) == ERROR_CODE.BAD_REQUEST_ERROR:
                raise BadRequestError(msg)
            elif str.upper(code) == ERROR_CODE.GATEWAY_ERROR:
                raise GatewayError(msg)
            elif str.upper(code) == ERROR_CODE.SERVER_ERROR: # nosemgrep : python.lang.maintainability.useless-ifelse.useless-if-body
                raise ServerError(msg)
            else:
                raise ServerError(msg)

    def get(self, path, params, **options):
        """
        Parses GET request options and dispatches a request
        """
        return self.request('get', path, params=params, **options)

    def post(self, path, data, **options):
        """
        Parses POST request options and dispatches a request
        """
        data, options = self._update_request(data, options)
        return self.request('post', path, data=data, **options)

    def patch(self, path, data, **options):
        """
        Parses PATCH request options and dispatches a request
        """
        data, options = self._update_request(data, options)
        return self.request('patch', path, data=data, **options)

    def delete(self, path, data, **options):
        """
        Parses DELETE request options and dispatches a request
        """
        data, options = self._update_request(data, options)
        return self.request('delete', path, data=data, **options)

    def put(self, path, data, **options):
        """
        Parses PUT request options and dispatches a request
        """
        data, options = self._update_request(data, options)
        return self.request('put', path, data=data, **options)

    def _update_request(self, data, options):
        """
        Updates The resource data and header options
        """
        data = json.dumps(data)

        if 'headers' not in options:
            options['headers'] = {}

        options['headers'].update({'Content-type': 'application/json'})

        return data, options

    def generate_payment_link(self,data):
        if data.get('success')==True:
            url = "{}{}/{}?key={}".format(self.endpoint, URL.PAYMENT_LINK,data['data']['session_id'],self.publishable_key)
            return url
        else:
            return data
    





import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientFundAccount(ClientTestCase):

    def setUp(self):
        super(TestClientFundAccount, self).setUp()
        self.base_url = '{}/fund_accounts'.format(self.base_url)

    @responses.activate
    def test_all(self):
        result = mock_file('fund_account_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.fund_account.all(), result)

    @responses.activate
    def test_create(self):
        param = {
            "customer_id": "cust_IEfAt3ruD4OEzo",
            "account_type":"bank_account",
            "bank_account":{
                "name":"Gaurav Kumar",
                "account_number":"11214311215411",
                "ifsc":"HDFC0000053"
            }
        }
        result = mock_file('fund_account_collection')
        url = self.base_url
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.fund_account.create(param), result)
 


import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientInvoice(ClientTestCase):

    def setUp(self):
        super(TestClientInvoice, self).setUp()
        self.base_url = '{}/invoices'.format(self.base_url)

    @responses.activate
    def test_invoice_fetch_all(self):
        result = mock_file('invoice_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.invoice.all(), result)

    @responses.activate
    def test_invoice_fetch_all_with_options(self):
        count = 1
        result = mock_file('invoice_collection_with_one_invoice')
        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.all({'count': count}), result)

    @responses.activate
    def test_invoice_fetch(self):
        result = mock_file('fake_invoice')
        url = '{}/{}'.format(self.base_url, 'fake_invoice_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.fetch('fake_invoice_id'), result)

    @responses.activate
    def test_invoice_create(self):
        init = mock_file('init_invoice')
        result = mock_file('fake_invoice')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.create(init), result)

    @responses.activate
    def test_invoice_notify_by(self):
        medium = "sms"
        result = mock_file('fake_invoice_notify_by')
        url = '{}/{}/notify_by/{}'.format(self.base_url, 'fake_invoice_id', medium)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.notify_by('fake_invoice_id', medium=medium), result)

    @responses.activate
    def test_invoice_cancel(self):
        result = mock_file('fake_invoice_cancel')
        url = '{}/{}/cancel'.format(self.base_url, 'fake_invoice_id')
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.cancel('fake_invoice_id'), result)

    @responses.activate
    def test_invoice_delete(self):
        result = mock_file('fake_invoice_delete')
        url = '{}/{}'.format(self.base_url, 'fake_invoice_id')
        responses.add(responses.DELETE, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.delete('fake_invoice_id'), result)

    @responses.activate
    def test_invoice_issue(self):
        result = mock_file('fake_invoice')
        url = '{}/{}/issue'.format(self.base_url, 'fake_invoice_id')
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.issue('fake_invoice_id'), result)

    @responses.activate
    def test_invoice_edit(self):
        init = mock_file('init_invoice_edit')
        result = mock_file('fake_invoice_edit')
        url = '{}/{}'.format(self.base_url, 'fake_invoice_id')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.invoice.edit('fake_invoice_id', init), result)



import responses
import json

from .helpers import ClientTestCase
from thawani.errors import BadRequestError, GatewayError, ServerError


class TestClientError(ClientTestCase):

    def setUp(self):
        super(TestClientError, self).setUp()
        self.base_url = '{}/payments'.format(self.base_url)

    @responses.activate
    def test_payment_with_invalid_options(self):
        count = 10000
        result = {
            'error':
            {
                'field': 'count',
                'code': 'BAD_REQUEST_ERROR',
                'description': 'The count may not be greater than 100.'
            }
        }

        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=400, body=json.dumps(result),
                      match_querystring=True)
        self.assertRaises(
            BadRequestError,
            self.client.payment.all,
            {'count': count})

    @responses.activate
    def test_gateway_error(self):
        count = 10
        result = {
            'error':
            {
                'code': 'GATEWAY_ERROR',
                'description': 'Payment processing failed due to error at bank/wallet gateway'
            }
        }

        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=504, body=json.dumps(result),
                      match_querystring=True)
        self.assertRaises(
            GatewayError,
            self.client.payment.all,
            {'count': count})

    @responses.activate
    def test_server_error(self):
        count = 10
        result = {
            'error':
            {
                'code': 'SERVER_ERROR',
                'description': 'The server encountered an error. The incident has been reported to admins.'
            }
        }

        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=500, body=json.dumps(result),
                      match_querystring=True)
        self.assertRaises(
            ServerError,
            self.client.payment.all,
            {'count': count})

    @responses.activate
    def test_unknown_error(self):
        count = 10
        result = {
            'error':
            {
                'code': 'UNKNOWN_ERROR',
                'description': 'No Description'
            }
        }

        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=500, body=json.dumps(result),
                      match_querystring=True)
        self.assertRaises(
            ServerError,
            self.client.payment.all,
            {'count': count})



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientQrcode(ClientTestCase):

    def setUp(self):
        super(TestClientQrcode, self).setUp()
        self.base_url = '{}/payments/qr_codes'.format(self.base_url)
        self.plan_id = 'qr_IAgePI1GuSMTFN'

    @responses.activate
    def test_qrcode_all(self):
        result = mock_file('qrcode_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.qrcode.all(), result)

    @responses.activate
    def test_qrcode_fetch(self):
        result = mock_file('fake_qrcode')
        url = '{}/{}'.format(self.base_url, 'fake_qrcode_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.qrcode.fetch('fake_qrcode_id'), result)

    @responses.activate
    def test_qrcode_create(self):
        init = {
            "type": "upi_qr",
            "name": "Store_1",
            "usage": "single_use",
            "fixed_amount": 1,
            "payment_amount": 300,
            "description": "For Store 1",
            "customer_id": "cust_HKsR5se84c5LTO",
            "close_by": 1681615838,
            "notes": {
                "purpose": "Test UPI QR code notes"
            }
        }
        result = mock_file('fake_qrcode')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.qrcode.create(init), result)

    @responses.activate
    def test_qrcode_fetch_all_payment(self):
        result = mock_file('qrcode_payments_collection')
        url = '{}/{}/payments'.format(self.base_url, 'fake_qrcode_id')
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.qrcode.fetch_all_payments('fake_qrcode_id'), result)

    @responses.activate
    def test_qrcode_close(self):
        result = mock_file('fake_qrcode')
        url = '{}/{}/close'.format(self.base_url, 'fake_qrcode_id')
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.qrcode.close('fake_qrcode_id'), result)


import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientPayment(ClientTestCase):

    def setUp(self):
        super(TestClientPayment, self).setUp()
        self.base_url = '{}/payments'.format(self.base_url)

    @responses.activate
    def test_payment_all(self):
        result = mock_file('payment_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.payment.all(), result)

    @responses.activate
    def test_payment_all_with_options(self):
        count = 1
        result = mock_file('payment_collection_with_one_payment')
        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.all({'count': count}), result)

    @responses.activate
    def test_payment_fetch(self):
        result = mock_file('fake_payment')
        url = '{}/{}'.format(self.base_url, self.payment_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.fetch('fake_payment_id'), result)

    @responses.activate
    def test_payment_capture(self):
        result = mock_file('fake_captured_payment')
        url = '{}/{}/capture'.format(self.base_url, self.payment_id)
        responses.add(responses.POST, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.payment.capture(self.payment_id,
                                                     amount=5100), result)

    @responses.activate
    def test_refund_create(self):
        result = mock_file('fake_refund')
        url = '{}/{}/refund'.format(self.base_url, self.payment_id)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.refund(self.payment_id, 2000),
                         result)

    @responses.activate
    def test_transfer(self):
        param = {
            'transfers': {
                'currency': {
                    'amount': 100,
                    'currency': 'INR',
                    'account': 'dummy_acc'
                }
            }
        }
        result = mock_file('transfers_collection_with_payment_id')
        url = '{}/{}/transfers'.format(self.base_url, self.payment_id)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.transfer(self.payment_id, param),
                         result)

    @responses.activate
    def test_transfer_fetch(self):
        result = mock_file('transfers_collection_with_payment_id')
        url = '{}/{}/transfers'.format(self.base_url, self.payment_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.transfers(self.payment_id), result)

    @responses.activate
    def test_bank_transfer_fetch(self):
        result = mock_file('fake_bank_transfer')
        url = '{}/{}/bank_transfer'.format(self.base_url, self.payment_id)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.payment.bank_transfer(self.payment_id)
        self.assertEqual(response['virtual_account_id'], 'va_8J2ny4Naokqbpe')
        self.assertEqual(response['payment_id'], self.payment_id)

    @responses.activate
    def test_upi_transfer_fetch(self):
        result = mock_file('fake_upi_transfer')
        url = '{}/{}/upi_transfer'.format(self.base_url, self.payment_id)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.payment.upi_transfer(self.payment_id)
        self.assertEqual(response['virtual_account_id'], 'va_8J2ny4Naokqbpf')
        self.assertEqual(response['payment_id'], self.payment_id)
    
    @responses.activate
    def test_payment_refund(self):
        init = {
            "amount": "100"
         }
        result = mock_file('fake_refund')
        url = '{}/{}/refund'.format(self.base_url, 'fake_refund_id')
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.refund('fake_refund_id',init), result)            

    @responses.activate
    def test_payment_fetch_multiple_refund(self):
        result = mock_file('refund_collection')
        url = "{}/{}/refunds".format(self.base_url, 'fake_payment_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.fetch_multiple_refund(self.payment_id), result)    

    @responses.activate
    def test_payment_fetch_refund_id(self):
        result = mock_file('refund_collection')
        url = "{}/{}/refunds/{}".format(self.base_url, 'fake_payment_id', 'fake_refund_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                    match_querystring=True)
        self.assertEqual(self.client.payment.fetch_refund_id('fake_payment_id', 'fake_refund_id'), result)    

    @responses.activate
    def test_payment_edit(self):
        param = {
                  "notes": {
                    "key1": "value3",
                    "key2": "value2"
                   }
               }

        result = mock_file('edit_payment')
        url = '{}/{}'.format(self.base_url, 'dummy_id')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.edit('dummy_id', param), result) 


    @responses.activate
    def test_fetch_card_detail(self):
        result = mock_file('fake_card_detail_payment')
        url = '{}/{}/card'.format(self.base_url, 'dummy_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.fetchCardDetails('dummy_id'), result)

    @responses.activate
    def test_fetch_downtimes(self):
        result = mock_file('fake_card_detail_payment')
        url = '{}/{}'.format(self.base_url, 'downtimes')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.fetchDownTime(), result) 

    @responses.activate
    def test_fetch_downtime_by_id(self):
        result = mock_file('fake_card_detail_payment')
        url = '{}/downtimes/{}'.format(self.base_url, 'dummy_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.fetchDownTimeById('dummy_id'), result)                   
    
    @responses.activate
    def test_payment_json(self):
        param = {
            "amount": "500",
            "currency": "INR",
            "email": "gaurav.kumar@example.com",
            "contact": "9123456789",
            "order_id": "order_IfCjbAb066hM9i",
            "method": "upi",
            "card": {
                "number": "4854980604708430",
                "cvv": "123",
                "expiry_month": "12",
                "expiry_year": "21",
                "name": "Gaurav Kumar"
            }
        }
        result = mock_file('fake_payment_json')
        url = "{}/create/{}".format(self.base_url, 'json')
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment.createPaymentJson(param), result)
    
    @responses.activate
    def test_createRecurring(self):
        init = mock_file('init_create_recurring')
        result = mock_file('fake_create_recurring')
        url = "{}/{}/recurring".format(self.base_url,'create')
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.payment.createRecurring(init), result)

    @responses.activate
    def test_otpGenerate(self):
        result = mock_file('fake_otp_generate')
        url = "{}/{}/otp_generate".format(self.base_url,'dummy_id')
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.payment.otpGenerate('dummy_id'), result)

    @responses.activate
    def test_otpSubmit(self):
        param = {
            "otp": "123456"
        }

        result = mock_file('fake_otp_submit')
        url = "{}/{}/otp/submit".format(self.base_url,'dummy_id')
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.payment.otpSubmit('dummy_id',param), result)        
    
    @responses.activate
    def test_otpResend(self):
        result = mock_file('fake_otp_resend')
        url = "{}/{}/otp/resend".format(self.base_url,'dummy_id')
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.payment.otpResend('dummy_id'), result)


from .base import Resource
from ..constants.url import URL


class Addon(Resource):
    def __init__(self, client=None):
        super(Addon, self).__init__(client)
        self.base_url = URL.ADDON_URL

    def fetch(self, addon_id, data={}, **kwargs):
        """"
        Fetch addon for given Id

        Args:
            addon_id : Id for which addon object has to be retrieved

        Returns:
            addon dict for given subscription Id
        """
        return super(Addon, self).fetch(addon_id, data, **kwargs)

    def delete(self, addon_id, data={}, **kwargs):
        """
        Delete addon for given id

        Args:
            addon_id : Id for which addon object has to be deleted
        """
        url = '{}/{}'.format(self.base_url, addon_id)

        return self.delete_url(url, data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch all Add-ons
        Returns:
            Dictionary of Add-ons
        """
        return super(Addon, self).all(data, **kwargs)    



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientRefund(ClientTestCase):

    def setUp(self):
        super(TestClientRefund, self).setUp()
        self.base_url = '{}/refunds'.format(self.base_url)

    @responses.activate
    def test_refund_all(self):
        result = mock_file('refund_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.refund.all(), result)

    @responses.activate
    def test_refund_fetch(self):
        result = mock_file('fake_refund')
        url = '{}/{}'.format(self.base_url, self.refund_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.refund.fetch(self.refund_id), result)

    @responses.activate
    def test_refund_create(self):
        init = {'payment_id': self.payment_id}
        result = mock_file('fake_refund')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.refund.create(init), result)

    @responses.activate
    def test_refund_edit(self):
        param = {
                   "notes": {
                   "notes_key_1":"Beam me up Scotty.",
                   "notes_key_2":"Engage"
                    }
                }
        result = mock_file('fake_refund')
        url = "{}/{}".format(self.base_url,'rfnd_DfjjhJC6eDvUAi')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.refund.edit('rfnd_DfjjhJC6eDvUAi', param), result)



from .base import Resource
from ..constants.url import URL


class Checkout(Resource):
    def __init__(self, client=None):
        super(Checkout, self).__init__(client)
        self.base_url = URL.CHECKOUT_SESSION_URL

    def fetch(self, session_id, data={}, **kwargs):
        """"
        Fetch checkout session by session_id

        Args:
            session_id : Id for which session object has to be retrieved

        Returns:
            Order dict for given session Id
        """
        return super(Checkout, self).fetch(session_id, data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create Checkout from given dict

        Returns:
            Checkout Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)
    
    
    def fetch_session_by_client_id(self, client_id=None, **kwargs):
        """"
        Create Checkout from given dict

        Returns:
            Checkout Dict which was created
        """        
        self.base_url = URL.CHECKOUT_SESSION_BY_REFERENCE_ID
        return super(Checkout, self).fetch(client_id, {}, **kwargs)
 
    
    def all(self, data={}, **kwargs):
        """"
        Fetch all session

        Returns:
            Dictionary of Checkout data
        """
        return super(Checkout, self).all(data, **kwargs)


from .client import Client
from .resources import Order
from .resources import Payment
from .resources import Refund
from .resources import Invoice
from .resources import PaymentLink
from .resources import Customer
from .resources import Checkout
from .resources import Card
from .resources import Token
from .resources import Transfer
from .resources import VirtualAccount
from .resources import Addon
from .resources import Subscription
from .resources import RegistrationLink
from .resources import Plan
from .resources import Settlement
from .resources import Item
from .resources import Qrcode
from .resources import FundAccount
from .utility import Utility
from .constants import ERROR_CODE
from .constants import HTTP_STATUS_CODE

__all__ = [
        'Payment',
        'Refund',
        'Order',
        'Client',
        'Invoice',
        'PaymentLink',
        'Utility',
        'Customer',
        'Card',
        'Token',
        'Transfer',
        'VirtualAccount',
        'Addon',
        'Subscription',
        'RegistrationLink',
        'Plan',
        'FundAccount',
        'Settlement',
        'Item',
        'Qrcode',
        'HTTP_STATUS_CODE',
        'ERROR_CODE',
        'Checkout',
]



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientPayment(ClientTestCase):

    def setUp(self):
        super(TestClientPayment, self).setUp()
        self.base_url = '{}/payments'.format(self.base_url)
        self.secondary_base_url = '{}/payments'.format(self.secondary_url)

    @responses.activate
    def test_payment_primary_url(self):
        result = mock_file('payment_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.payment.all(), result)

    @responses.activate
    def test_payment_secondary_url(self):
        result = mock_file('payment_collection')
        url = self.secondary_base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.secondary_client.payment.all(), result)

    @responses.activate
    def test_payment_with_headers(self):
        result = mock_file('payment_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.payment.all(
            headers={'Content-type': 'text'}),
            result)



from .base import Resource
from ..constants.url import URL


class RegistrationLink(Resource):
    def __init__(self, client=None):
        super(RegistrationLink, self).__init__(client)
        self.base_url = URL.REGISTRATION_LINK_URL

    def create(self, data={}, **kwargs):
        """"
        Create a Registration Link
        Args:
            customer : Details of the customer to whom the registration link will be sent.
            type* : In this case the value is link.
            currency* : Currency used in Order
            amount* : Amount of Order
            description : The count may not be greater than 100.
            subscription_registration : Details of the authorization payment.
            notes : A key-value pair
        
        Returns:
            {"success": true}
        """
        url = "{}/{}".format(self.base_url, 'auth_links')
        return self.post_url(url, data, **kwargs)


import responses

from .helpers import mock_file, ClientTestCase
from thawani.errors import SignatureVerificationError


class TestClientValidator(ClientTestCase):

    def setUp(self):
        super(TestClientValidator, self).setUp()

    @responses.activate
    def test_verify_payment_signature(self):
        sig = 'b2335e3b0801106b84a7faff035df56ecffde06918c9ddd1f0fafbb37a51cc89'
        parameters = {}
        parameters['thawani_order_id'] = 'fake_order_id'
        parameters['thawani_payment_id'] = 'fake_payment_id'
        parameters['thawani_signature'] = sig

        self.assertEqual(
             self.client.utility.verify_payment_signature(parameters),
             True)

    @responses.activate
    def test_subscription_payment_signature(self):
        sig = '601f383334975c714c91a7d97dd723eb56520318355863dcf3821c0d07a17693'
        parameters = {}
        parameters['thawani_subscription_id'] = 'sub_ID6MOhgkcoHj9I'
        parameters['thawani_payment_id'] = 'pay_IDZNwZZFtnjyym'
        parameters['thawani_signature'] = sig
        parameters['secret'] = 'EnLs21M47BllR3X8PSFtjtbd'

        self.assertEqual(
             self.client.utility.verify_subscription_payment_signature(parameters),
             True)

    @responses.activate
    def test_verify_payment_link_signature(self):
        sig = '07ae18789e35093e51d0a491eb9922646f3f82773547e5b0f67ee3f2d3bf7d5b'
        parameters = {}
        parameters['thawani_payment_id'] = 'pay_IH3d0ara9bSsjQ'
        parameters['payment_link_id'] = 'plink_IH3cNucfVEgV68'
        parameters['payment_link_reference_id'] = 'TSsd1989'
        parameters['payment_link_status'] = 'paid'
        parameters['thawani_signature'] = sig
        parameters['secret'] = 'EnLs21M47BllR3X8PSFtjtbd'
       # x = self.client.utility.verify_payment_link_signature(parameters)

        self.assertEqual(
             self.client.utility.verify_payment_link_signature(parameters),
             True)

    @responses.activate
    def test_verify_payment_signature_with_exception(self):
        parameters = {}
        parameters['thawani_order_id'] = 'fake_order_id'
        parameters['thawani_payment_id'] = 'fake_payment_id'
        parameters['thawani_signature'] = 'test_signature'

        self.assertRaises(
            SignatureVerificationError,
            self.client.utility.verify_payment_signature,
            parameters)

    @responses.activate
    def test_verify_webhook_signature(self):
        secret = self.client.auth[1]
        sig = 'd60e67fd884556c045e9be7dad57903e33efc7172c17c6e3ef77db42d2b366e9'
        body = mock_file('fake_payment_authorized_webhook')

        self.assertEqual(
             self.client.utility.verify_webhook_signature(body, sig, secret),
             True)

    @responses.activate
    def test_verify_webhook_signature_with_exception(self):
        secret = self.client.auth[1]
        sig = 'test_signature'
        body = ''

        self.assertRaises(
            SignatureVerificationError,
            self.client.utility.verify_webhook_signature,
            body,
            sig,
            secret)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientTransfer(ClientTestCase):

    def setUp(self):
        super(TestClientTransfer, self).setUp()
        self.payment_url = '{}/payments'.format(self.base_url)
        self.base_url = '{}/transfers'.format(self.base_url)

    @responses.activate
    def test_transfer_all(self):
        result = mock_file('transfers_collection')
        url = self.base_url
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.transfer.all(), result)

    @responses.activate
    def test_transfer_all_with_payment_id(self):
        result = mock_file('transfers_collection_with_payment_id')
        url = "{}/dummy_payment/transfers".format(self.payment_url)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.transfer.all(
            {'payment_id': 'dummy_payment'}),
            result)

    @responses.activate
    def test_transfer_fetch(self):
        result = mock_file('fake_transfer')
        url = '{}/{}'.format(self.base_url, 'fake_transfer_id')
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.transfer.fetch('fake_transfer_id'), result)

    @responses.activate
    def test_transfer_create(self):
        init = mock_file('init_transfer')
        result = mock_file('fake_transfer')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.transfer.create(init), result)

    @responses.activate
    def test_transfer_edit(self):
        param = {'on_hold': False}
        result = mock_file('fake_transfer')
        url = "{}/dummy_id".format(self.base_url)
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.transfer.edit('dummy_id', param), result)

    @responses.activate
    def test_transfer_reversal(self):
        result = mock_file('fake_reversal')
        url = "{}/dummy_id/reversals".format(self.base_url)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.transfer.reverse('dummy_id'), result)

    @responses.activate
    def test_transfer_reversal_fetch(self):
        result = mock_file('reversal_collection')
        url = "{}/dummy_id/reversals".format(self.base_url)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.transfer.reversals('dummy_id'), result)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientSettlement(ClientTestCase):

    def setUp(self):
        super(TestClientSettlement, self).setUp()
        self.base_url = '{}/settlements'.format(self.base_url)

    @responses.activate
    def test_settlement_fetch_all(self):
        result = mock_file('settlement_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.settlement.all(), result)

    @responses.activate
    def test_settlement_fetch_all_with_options(self):
        count = 1
        result = mock_file('settlement_collection_with_one_settlement')
        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.settlement.all({'count': count}), result)

    @responses.activate
    def test_settlement_fetch(self):
        result = mock_file('fake_settlement')
        url = '{}/{}'.format(self.base_url, self.settlement_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.settlement.fetch(self.settlement_id), result)

    @responses.activate
    def test_settlement_report(self):
        result = mock_file('settlement_collection')
        url = "{}/recon/{}".format(self.base_url, 'combined')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.settlement.report(), result)    

    @responses.activate
    def test_settlement_create_ondemand_settlement(self):
        init = {
                 "amount": 1221,
                 "description": "Need this to make vendor",
                 "notes": {
                     "notes_key_1": "Tea, Earl Grey, Hot",
                     "notes_key_2": "Tea, Earl Grey… decaf."
                 }
               }
        result = mock_file('init_settlement')
        url = "{}/{}".format(self.base_url,"ondemand")
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.settlement.create_ondemand_settlement(init), result)     

    @responses.activate
    def test_settlement_fetch_all_ondemand_settlement(self):
        result = mock_file('settlement_collection')
        url = "{}/{}".format(self.base_url,"ondemand")
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.settlement.fetch_all_ondemand_settlement(), result)        

    @responses.activate
    def test_settlement_fetch_ondemand_settlement_id(self):
        result = mock_file('init_settlement')
        url = "{}/ondemand/{}".format(self.base_url, 'fake_settlement_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.settlement.fetch_ondemand_settlement_id('fake_settlement_id'), result)


class URL(object):
    SANDBOX_URL = 'https://uatcheckout.thawani.om/api/v1'
    SANDBOX_ENDPOINT = 'https://uatcheckout.thawani.om'
    PROD_ENDPOINT = 'https://checkout.thawani.om'
    PROD_URL = 'https://checkout.thawani.om/api/v1'




    ORDER_URL = "/orders"
    PAYMENT_INTENT   ="/payment_intents"
    INVOICE_URL = "/invoices"
    PAYMENT_LINK_URL = "/payment_links"
    PAYMENTS_URL = "/payments"
    REFUNDS_URL = "/refunds"
    CARD_URL = "/cards"
    CUSTOMER_URL = "/customers"
    TRANSFER_URL = "/transfers"
    VIRTUAL_ACCOUNT_URL = "/virtual_accounts"
    SUBSCRIPTION_URL = "/subscriptions"
    ADDON_URL = "/addons"
    PLAN_URL = "/plans"
    SETTLEMENT_URL = "/settlements"
    ITEM_URL = "/items"
    QRCODE_URL = "/payments/qr_codes"
    REGISTRATION_LINK_URL = "/subscription_registration"
    FUND_ACCOUNT_URL = "/fund_accounts"
    CHECKOUT_SESSION_URL="/checkout/session"
    PAYMENT_METHOD_URL="/payment_methods"
    PAYMENT_LINK="/pay"
    CHECKOUT_SESSION_BY_REFERENCE_ID="/checkout/reference"
    



import responses
import json
from .helpers import mock_file, ClientTestCase


class TestClientVirtualAccount(ClientTestCase):

    def setUp(self):
        super(TestClientVirtualAccount, self).setUp()
        self.base_url = '{}/virtual_accounts'.format(self.base_url)
        self.fake_virtual_account_id = 'va_4xbQrmEoA5WJ0G'

    @responses.activate
    def test_virtual_accounts_create(self):
        init = mock_file('init_virtual_accounts')
        result = mock_file('fake_virtual_accounts')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=result,
                      match_querystring=True)

        response = self.client.virtual_account.create(init)
        self.assertEqual(response['description'], 'First Virtual Account')
        self.assertEqual(response['status'], 'active')
        self.assertEqual(response['amount_paid'], 0)
        self.assertIsNotNone(response['receivers'][0]['account_number'])

    @responses.activate
    def test_virtual_accounts_all(self):
        result = mock_file('virtual_accounts_collection')
        url = self.base_url
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.all()
        self.assertEqual(response['entity'], 'collection')
        self.assertEqual(response['count'], 2)
        self.assertEqual(len(response['items']), 2)

    @responses.activate
    def test_virtual_accounts_all_with_options(self):
        count = 1
        result = mock_file('virtual_accounts_collection_with_one_item')
        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.all({'count': count})
        self.assertEqual(response['entity'], 'collection')
        self.assertEqual(response['count'], count)

    @responses.activate
    def test_virtual_accounts_fetch(self):
        result = mock_file('fake_virtual_accounts')
        url = '{}/{}'.format(self.base_url, self.fake_virtual_account_id)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.fetch(
            self.fake_virtual_account_id)
        self.assertEqual(response['id'], self.fake_virtual_account_id)
        self.assertEqual(response['entity'], 'virtual_account')
        self.assertIsNotNone(response['receivers'][0]['account_number'])

    @responses.activate
    def test_virtual_accounts_close(self):
        result = mock_file('fake_virtual_accounts_closed')
        url = '{}/{}/close'.format(self.base_url, self.fake_virtual_account_id)
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.close(
            self.fake_virtual_account_id)
        self.assertEqual(response['id'], self.fake_virtual_account_id)
        self.assertEqual(response['entity'], 'virtual_account')
        self.assertEqual(response['status'], 'closed')

    @responses.activate
    def test_virtual_accounts_payments(self):
        result = mock_file('payment_collection')
        url = '{}/{}/payments'.format(
            self.base_url,
            self.fake_virtual_account_id)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.payments(
            self.fake_virtual_account_id)
        self.assertEqual(response['entity'], 'collection')
        self.assertEqual(response['count'], 2)
        self.assertEqual(len(response['items']), 2)
        self.assertEqual(response['items'][0]['entity'], 'payment')
    
    @responses.activate
    def test_virtual_add_receiver(self):
        init =  {
                  "types": ["vpa"],
                  "vpa": {
                      "descriptor": "gaurikumar"
                   }
                }
        result = mock_file('fake_virtual_accounts_closed')
        url = "{}/{}/receivers".format(self.base_url, self.fake_virtual_account_id)
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.add_receiver(
            self.fake_virtual_account_id, init)
        self.assertEqual(response['id'], self.fake_virtual_account_id)
        self.assertEqual(response['entity'], 'virtual_account')

    @responses.activate
    def test_virtual_add_allowed_player(self):
        init =  {
                  "type": "bank_account",
                  "bank_account": {
                      "ifsc": "UTIB0000013",
                      "account_number": "914010012345679"
                    }
                }
        result = mock_file('fake_virtual_accounts_closed')
        url = "{}/{}/allowed_payers".format(self.base_url, self.fake_virtual_account_id)
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=result,
                      match_querystring=True)

        response = self.client.virtual_account.add_allowed_player(
            self.fake_virtual_account_id, init)   
        self.assertEqual(response['id'], self.fake_virtual_account_id)
        self.assertEqual(response['entity'], 'virtual_account')

   
    @responses.activate
    def test_virtual_delete_allowed_player(self):
        result = mock_file('fake_delete_allowed_payer')
        url = "{}/{}/allowed_payers/{}".format(self.base_url, self.fake_virtual_account_id, 'fake_allowed_player_id')
        responses.add(responses.DELETE,
                      url,
                      status=204,
                      body=result,
                      match_querystring=True)
        response = self.client.virtual_account.delete_allowed_player(
        self.fake_virtual_account_id, 'fake_allowed_player_id');   
        self.assertEqual(response, result)
          
   


import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientRegistrationLink(ClientTestCase):

    def setUp(self):
        super(TestClientRegistrationLink, self).setUp()
        self.base_url = '{}/subscription_registration/{}'.format(self.base_url, 'auth_links')

    @responses.activate
    def test_create(self):
        param = mock_file('init_registration_link')
        result = mock_file('fake_registration_link')
        url = self.base_url
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.registration_link.create(param), result)



from .payment import Payment
from .refund import Refund
from .order import Order
from .invoice import Invoice
from .payment_link import PaymentLink
from .customer import Customer
from .card import Card
from .token import Token
from .transfer import Transfer
from .virtual_account import VirtualAccount
from .addon import Addon
from .plan import Plan
from .subscription import Subscription
from .qrcode import Qrcode
from .registration_link import RegistrationLink
from .settlement import Settlement
from .item import Item
from .fund_account import FundAccount
from .checkout import Checkout
from .payment_method import PaymentMethod
from .payment_intent import PaymentIntent
__all__ = [
    'Payment',
    'Refund',
    'Order',
    'Invoice',
    'PaymentLink',
    'Customer',
    'Card',
    'Token',
    'Transfer',
    'VirtualAccount',
    'Addon',
    'Plan',
    'Subscription',
    'RegistrationLink',
    'Settlement',
    'Item',
    'QrCode',
    'FundAccount',
    'Checkout',
    'PaymentMethod',
    "PaymentIntent"
]



from .base import Resource
from ..constants.url import URL


class Settlement(Resource):
    def __init__(self, client=None):
        super(Settlement, self).__init__(client)
        self.base_url = URL.SETTLEMENT_URL

    def all(self, data={}, **kwargs):
        """"
        Fetch all Settlement entities

        Returns:
            Dictionary of Settlement data
        """
        return super(Settlement, self).all(data, **kwargs)

    def fetch(self, settlement_id, data={}, **kwargs):
        """"
        Fetch Settlement data for given Id

        Args:
            settlement_id : Id for which settlement object has to be retrieved

        Returns:
            settlement dict for given settlement id
        """
        return super(Settlement, self).fetch(settlement_id, data, **kwargs)

    def report(self, data={}, **kwargs):
        """"
        Settlement report for a month

        Returns:
            settlement dict
        """
        url = "{}/recon/{}".format(self.base_url, 'combined')
        return self.get_url(url, data, **kwargs)    

    def create_ondemand_settlement(self, data={}, **kwargs):
        """
        create Ondemand Settlemententity

        Returns:
            settlement dict which was created
        """
        url = "{}/{}".format(self.base_url,"ondemand")
        return self.post_url(url, data, **kwargs)

    def fetch_all_ondemand_settlement(self, data={}, **kwargs):
        """
        create Ondemand Settlemententity

        Returns:
            settlement dict which was created
        """
        url = "{}/{}".format(self.base_url,"ondemand")
        return self.get_url(url, data, **kwargs)

    def fetch_ondemand_settlement_id(self, settlement_id, data={}, **kwargs):
        """
        fetch Ondemand Settlement by Id 

        Returns:
            settlement dict for given settlement id
        """
        url = "{}/ondemand/{}".format(self.base_url, settlement_id)
        return self.get_url(url, data, **kwargs)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientUserAgent(ClientTestCase):

    def setUp(self):
        super(TestClientUserAgent, self).setUp()
        app_details = json.loads(mock_file('fake_app_details'))
        self.client.set_app_details(app_details)
        self.base_url = "{}/payments".format(self.base_url)

    @responses.activate
    def test_payment_all(self):
        result = mock_file('payment_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.payment.all(), result)



from .base import Resource
from ..constants.url import URL


class PaymentMethod(Resource):
    def __init__(self, client=None):
        super(PaymentMethod, self).__init__(client)
        self.base_url = URL.PAYMENT_METHOD_URL

    def all(self, data={}, **kwargs):
        """"
        Fetch all customer

        Returns:
            Dictionary of Customers data
        """
        return super(PaymentMethod, self).all(data, **kwargs)
    
    def delete(self, card_id, **kwargs):
        """"
        Delete an invoice
        You can delete an invoice which is in the draft state.

        Args:
            invoice_id : Id for delete the invoice
        Returns:
            The response is always be an empty array like this - []
        """
        url = "{}/{}".format(self.base_url, card_id)
        return self.delete_url(url, {}, **kwargs)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientPaymentLink(ClientTestCase):

    def setUp(self):
        super(TestClientPaymentLink, self).setUp()
        self.base_url = '{}/payment_links'.format(self.base_url)
    
    @responses.activate
    def test_payment_link_create(self):
        init = mock_file('init_payment_link')

        result = mock_file('fake_payment_link')
        url = '{}/{}'.format(self.base_url, 'fake_payment_link_id')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment_link.edit('fake_payment_link_id', init), result)


    @responses.activate
    def test_payment_link_edit(self):
        init = {
            "reference_id": "TS35",
            "expire_by": 1653347540,
            "reminder_enable":0,
            "notes":{
            "policy_name": "Jeevan Saral"
            }
        }

        result = mock_file('edit_payment_link')
        url = '{}/{}'.format(self.base_url, 'fake_payment_link_id')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment_link.edit('fake_payment_link_id', init), result)
    
    @responses.activate
    def test_payment_link_notifyBy(self):
        result = {"success": 1}

        url = "{}/{}/notify_by/{}".format(self.base_url, 'fake_payment_link_id', 'email')

        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment_link.notifyBy('fake_payment_link_id',medium='email'), result)

    @responses.activate
    def test_payment_link_all(self):
        result = mock_file('payment_link_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment_link.all(), result)

    @responses.activate
    def test_payment_all_fetch(self):
        result = mock_file('fake_payment_link')
        url = '{}/{}'.format(self.base_url, 'fake_payment_link_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment_link.fetch('fake_payment_link_id'),result)    

    @responses.activate
    def test_payment_link_cancel(self):
        result = mock_file('cancel_payment_link')

        url = "{}/{}/cancel".format(self.base_url, 'fake_payment_link_id')

        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.payment_link.cancel('fake_payment_link_id'), result)    



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientItem(ClientTestCase):

    def setUp(self):
        super(TestClientItem, self).setUp()
        self.base_url = '{}/items'.format(self.base_url)

    @responses.activate
    def test_item_all(self):
        result = mock_file('item_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.item.all(), result)

    @responses.activate
    def test_item_fetch(self):
        result = mock_file('item_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.item.all(), result)

    @responses.activate
    def test_item_create(self):
        result = mock_file('item_collection')
        url = '{}/{}'.format(self.base_url, 'fake_item_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.item.fetch('fake_item_id'), result)

    @responses.activate
    def test_item_delete(self):
        result = []
        url = '{}/{}'.format(self.base_url, 'fake_item_id')
        responses.add(responses.DELETE, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.item.delete('fake_item_id'), result)

    


from .base import Resource
from ..constants.url import URL
import json


class VirtualAccount(Resource):
    def __init__(self, client=None):
        super(VirtualAccount, self).__init__(client)
        self.base_url = URL.VIRTUAL_ACCOUNT_URL

    def all(self, data={}, **kwargs):
        """"
        Fetch all Virtual Account entities

        Returns:
            Dictionary of Virtual Account data
        """
        return super(VirtualAccount, self).all(data, **kwargs)

    def fetch(self, virtual_account_id, data={}, **kwargs):
        """"
        Fetch Virtual Account for given Id

        Args:
            virtual_account_id :
                Id for which Virtual Account object has to be retrieved

        Returns:
            Virtual Account dict for given Virtual Account Id
        """
        return super(VirtualAccount, self).fetch(
            virtual_account_id,
            data,
            **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create Virtual Account from given dict

        Args:
            Param for Creating Virtual Account

        Returns:
            Virtual Account dict
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def close(self, virtual_account_id, data={}, **kwargs):
        """"
        Close Virtual Account from given Id

        Args:
            virtual_account_id :
                Id for which Virtual Account objects has to be Closed
        """
        url = "{}/{}/close".format(self.base_url, virtual_account_id)
        return self.post_url(url, data, **kwargs)

    def payments(self, virtual_account_id, data={}, **kwargs):
        """"
        Fetch Payment for Virtual Account Id

        Args:
            virtual_account_id :
                Id for which Virtual Account objects has to be retrieved

        Returns:
            Payment dict for given Virtual Account Id
        """
        url = "{}/{}/payments".format(self.base_url, virtual_account_id)
        return self.get_url(url, data, **kwargs)

    def add_receiver(self, virtual_account_id, data={}, **kwargs):
        """"
        Add receiver to an existing virtual account

        Args:
            virtual_account_id :
                Id for which Virtual Account objects has to be Closed
        """
        url = "{}/{}/receivers".format(self.base_url, virtual_account_id)
        return self.post_url(url, data, **kwargs)

    def add_allowed_player(self, virtual_account_id, data={}, **kwargs):
        """"
        Add an Allowed Payer Account

        Args:
            virtual_account_id :
                Id for which Virtual Account objects has to be Closed
        """
        url = "{}/{}/allowed_payers".format(self.base_url, virtual_account_id)
        return self.post_url(url, data, **kwargs) 

    def delete_allowed_player(self, virtual_account_id, allowed_player_id, data={}, **kwargs):
        """"
        Delete an Allowed Payer Account

        Args:
            virtual_account_id :
                Id for which Virtual Account objects has to be Closed
        Returns:
            204        
        """
        url = "{}/{}/allowed_payers/{}".format(self.base_url, virtual_account_id, allowed_player_id)
        return self.delete_url(url, data, **kwargs)


import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientRefund(ClientTestCase):

    def setUp(self):
        super(TestClientRefund, self).setUp()
        self.base_url = '{}/cards'.format(self.base_url)

    @responses.activate
    def test_card_fetch(self):
        result = mock_file('fake_card')
        url = '{}/{}'.format(self.base_url, self.card_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.card.fetch(self.card_id), result)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientCustomer(ClientTestCase):

    def setUp(self):
        super(TestClientCustomer, self).setUp()
        self.base_url = '{}/customers'.format(self.base_url)

    @responses.activate
    def test_customer_fetch(self):
        result = mock_file('fake_customer')
        url = '{}/{}'.format(self.base_url, self.customer_id)

        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.customer.fetch(self.customer_id), result)

    @responses.activate
    def test_customer_create(self):
        init = mock_file('init_customer')
        result = mock_file('fake_customer')
        url = self.base_url
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.customer.create(init), result)

    @responses.activate
    def test_customer_edit(self):
        email = 'test@test.com'
        result = mock_file('fake_customer')
        url = '{}/{}'.format(self.base_url, self.customer_id)
        responses.add(responses.PUT,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.customer.edit(self.customer_id, email), result)
    
    @responses.activate
    def test_item_all(self):
        result = mock_file('customer_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.customer.all(), result)


from .base import Resource
from ..constants.url import URL
import warnings


class Payment(Resource):
    def __init__(self, client=None):
        super(Payment, self).__init__(client)
        self.base_url = URL.PAYMENTS_URL

    def fetch_all(self, data={}, **kwargs):  # pragma: no cover
        warnings.warn("Will be Deprecated in next release, use all",
                      DeprecationWarning)
        return self.all(data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch all Payment entities

        Returns:
            Dictionary of Payment data
        """
        return super(Payment, self).all(data, **kwargs)

    def fetch(self, payment_id, data={}, **kwargs):
        """"
        Fetch Payment for given Id

        Args:
            payment_id : Id for which payment object has to be retrieved

        Returns:
            Payment dict for given payment Id
        """
        return super(Payment, self).fetch(payment_id, data, **kwargs)

    def capture(self, payment_id, amount, data={}, **kwargs): # nosemgrep : python.lang.correctness.common-mistakes.default-mutable-dict.default-mutable-dict
        """"
        Capture Payment for given Id

        Args:
            payment_id : Id for which payment object has to be retrieved
            amount : Amount for which the payment has to be retrieved

        Returns:
            Payment dict after getting captured
        """
        url = "{}/{}/capture".format(self.base_url, payment_id)
        data['amount'] = amount
        return self.post_url(url, data, **kwargs)

    def refund(self, payment_id, amount, data={}, **kwargs):  # pragma: no cover # nosemgrep : python.lang.correctness.common-mistakes.default-mutable-dict.default-mutable-dict
        """"
        Refund Payment for given Id

        Args:
            payment_id : Id for which payment object has to be refunded
            amount : Amount for which the payment has to be refunded

        Returns:
            Payment dict after getting refunded
        """
        url = "{}/{}/refund".format(self.base_url, payment_id)
        data['amount'] = amount
        return self.post_url(url, data, **kwargs)

    def transfer(self, payment_id, data={}, **kwargs):
        """"
        Create Transfer for given Payment Id

        Args:
            payment_id : Id for which payment object has to be transferred

        Returns:
            Payment dict after getting transferred
        """
        url = "{}/{}/transfers".format(self.base_url, payment_id)
        return self.post_url(url, data, **kwargs)

    def transfers(self, payment_id, data={}, **kwargs):
        """"
        Fetches all transfer for given Payment Id

        Args:
            payment_id : Id for which all the transfers has to be fetched

        Returns:
            A collection (dict) of transfers
            items : The key containing a list of 'transfer' entities
        """
        url = "{}/{}/transfers".format(self.base_url, payment_id)
        return self.get_url(url, data, **kwargs)

    def bank_transfer(self, payment_id, data={}, **kwargs):
        """"
        Bank Transfer Entity for given Payment

        Args:
            payment_id : Id for which bank transfer entity has to be fetched

        Returns:
            Bank Transfer dict
        """
        url = "{}/{}/bank_transfer".format(self.base_url, payment_id)
        return self.get_url(url, data, **kwargs)

    def upi_transfer(self, payment_id, data={}, **kwargs):
        """"
        UPI Transfer Entity for given Payment

        Args:
            payment_id : Id for which upi transfer entity has to be fetched

        Returns:
            UPI Transfer dict
        """
        url = "{}/{}/upi_transfer".format(self.base_url, payment_id)
        return self.get_url(url, data, **kwargs)
    
    def refund(self, payment_id, data={}, **kwargs):
        """"
        Create a normal refund

        Returns:
            Payment dict after getting refund
        """
        url = "{}/{}/refund".format(self.base_url, payment_id)
        return self.post_url(url, data, **kwargs)

    def fetch_multiple_refund(self, payment_id, data={}, **kwargs):
        """"
        Fetch multiple refunds for a payment

        Returns:
            refunds dict
        """
        url = "{}/{}/refunds".format(self.base_url, payment_id)
        return self.get_url(url, data, **kwargs) 

    def fetch_refund_id(self, payment_id, refund_id, **kwargs):
        """"
        Fetch multiple refunds for a payment

        Returns:
            Refund dict
        """
        url = "{}/{}/refunds/{}".format(self.base_url, payment_id, refund_id)
        return self.get_url(url, {}, **kwargs)  
  
    def edit(self, payment_id, data={}, **kwargs):
        """"
         Update the Payment
        Args:
            data : Dictionary having keys using which order have to be edited
                'notes' : key value pair as notes
            
            Returns:
            Payment Dict which was edited
        """
        url = '{}/{}'.format(self.base_url, payment_id)

        return self.patch_url(url, data, **kwargs) 
    
    def fetchCardDetails(self, payment_id, **kwargs):
        """"
        Fetch Card Details of a Payment

        Args:
            payment_id : Id for which payment objects has to be retrieved

        Returns:
            Payment dict for given Order Id
        """
        url = "{}/{}/card".format(self.base_url, payment_id)
        return self.get_url(url, {}, **kwargs)

    def fetchDownTime(self, **kwargs):
        """"
        Fetch Card Details of a Payment

        Args:
            payment_id : Id for which payment objects has to be retrieved

        Returns:
            Payment dict for given Order Id
        """
        url = "{}/{}".format(self.base_url,'downtimes')
        return self.get_url(url, {}, **kwargs)        
    
    def fetchDownTimeById(self, downtime_id, **kwargs):
        """"
        Fetch Payment Downtime Details by ID

        Args:
            payment_id : Id for which payment objects has to be retrieved

        Returns:
            Payment dict for given Order Id
        """
        url = "{}/downtimes/{}".format(self.base_url, downtime_id)
        return self.get_url(url, {}, **kwargs)
    
    def createPaymentJson(self ,data={}, **kwargs):
        """"
        Create a Payment

        Args:
            payment_id : Id for which payment object has to be refunded
            amount : Amount for which the payment has to be refunded

        Returns:
            Payment Dict which was created
        """
        url = "{}/create/{}".format(self.base_url, 'json')

        return self.post_url(url, data, **kwargs)  

    def createRecurring(self, data={}, **kwargs):
        """"
        Create Recurring Payments
        Return:
            Recurring Payments dict
        """
        url = "{}/{}/recurring".format(self.base_url,'create')
        return self.post_url(url, data, **kwargs)

    def createUpi(self, data={}, **kwargs):
        """"
        Initiate a payment
        Return:
          Payments dict
        """
        url = "{}/create/{}".format(self.base_url,'upi')
        return self.post_url(url, data, **kwargs)

    def validateVpa(self, data={}, **kwargs):
        """"
        Validate the VPA
        Return:
          Payments dict
        """
        url = "{}/validate/{}".format(self.base_url,'vpa')
        return self.post_url(url, data, **kwargs)            

    def fetchPaymentMethods(self, **kwargs):
        """"
        Fetch payment methods
        Return:
          Payments dict
        """
        url = "/{}".format('methods')
        return self.get_url(url, {}, **kwargs)
    
    def otpGenerate(self, payment_id, data={}, **kwargs):
        """"
        Otp Generate

        Args:
            payment_id : Id for which upi transfer entity has to be fetched

        Returns:
            Otp Dict which was created
        """
        url = "{}/{}/otp_generate".format(self.base_url, payment_id)
        return self.post_url(url, data, **kwargs)

    def otpSubmit(self, payment_id, data={}, **kwargs):
        """"
        Otp Submit

        Args:
            payment_id : Id for which upi transfer entity has to be fetched

        Returns:
            Otp Dict which was created
        """
        url = "{}/{}/otp/submit".format(self.base_url, payment_id)
        return self.post_url(url, data, **kwargs)

    def otpResend(self, payment_id, data={}, **kwargs):
        """"
        Otp Resend

        Args:
            payment_id : Id for which upi transfer entity has to be fetched

        Returns:
            Otp Dict which was created
        """
        url = "{}/{}/otp/resend".format(self.base_url, payment_id)
        return self.post_url(url, data, **kwargs)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientOrder(ClientTestCase):

    def setUp(self):
        super(TestClientOrder, self).setUp()
        self.base_url = '{}/orders'.format(self.base_url)

    @responses.activate
    def test_order_all(self):
        result = mock_file('order_collection')
        url = self.base_url
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.order.all(), result)

    @responses.activate
    def test_order_all_with_options(self):
        count = 1
        result = mock_file('order_collection_with_one_order')
        url = '{}?count={}'.format(self.base_url, count)
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.order.all({'count': count}), result)

    @responses.activate
    def test_order_fetch(self):
        result = mock_file('fake_order')
        url = '{}/{}'.format(self.base_url, 'fake_order_id')
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.order.fetch('fake_order_id'), result)

    @responses.activate
    def test_order_payments(self):
        result = mock_file('fake_order')
        url = '{}/{}/payments'.format(self.base_url, 'fake_order_id')
        responses.add(responses.GET,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)

        self.assertEqual(self.client.order.payments('fake_order_id'), result)

    @responses.activate
    def test_order_create(self):
        init = mock_file('init_order')
        result = mock_file('fake_order')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.order.create(init), result)

    @responses.activate
    def test_order_edit(self):
        param = {
                  "notes": {
                    "key1": "value3",
                    "key2": "value2"
                   }
                }
        
        result = mock_file('edit_order')
        url = '{}/{}'.format(self.base_url, 'dummy_id')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.order.edit('dummy_id', param), result)



from .base import Resource
from ..constants.url import URL


class Item(Resource):
    def __init__(self, client=None):
        super(Item, self).__init__(client)
        self.base_url = URL.ITEM_URL
    
    def create(self, data={}, **kwargs):
        """"
        Create item

        Returns:
            Item Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def fetch(self, item_id, data={}, **kwargs):
        """"
        Fetch an Item

        Args:
            item_id : The id of the item to be fetched

        Returns:
            Item dict for given card Id
        """
        return super(Item, self).fetch(item_id, data, **kwargs)
    
    def all(self, data={}, **kwargs):
        """"
        Fetch all items

        Returns:
            Dictionary of Items data
        """
        return super(Item, self).all(data, **kwargs)

    def edit(self, item_id, data={}, **kwargs):
        """"
        Update an Item

        Returns:
            Item Dict which was edited
        """
        url = '{}/{}'.format(self.base_url, item_id)

        return self.patch_url(url, data, **kwargs)    
    
    def delete(self, item_id, **kwargs):
        """"
        Delete an Item

        Args:
            item_id : The id of the item to be deleted

        Returns:
            The response is always be an empty array like this - []
        """
        url = "{}/{}".format(self.base_url, item_id)
        return self.delete_url(url, {}, **kwargs)


from .base import Resource
from ..constants.url import URL


class Token(Resource):
    def __init__(self, client=None):
        super(Token, self).__init__(client)
        self.base_url = URL.CUSTOMER_URL

    def fetch(self, customer_id, token_id, data={}, **kwargs):
        """"
        Fetch Token for given Id and given customer Id

        Args:
            customer_id : Customer Id for which tokens have to be fetched
            token_id    : Id for which TOken object has to be fetched

        Returns:
            Token dict for given token Id
        """
        url = "{}/{}/tokens/{}".format(self.base_url, customer_id, token_id)
        return self.get_url(url, data, **kwargs)

    def all(self, customer_id, data={}, **kwargs):
        """"
        Get all tokens for given customer Id

        Args:
            customer_id : Customer Id for which tokens have to be fetched

        Returns:
            Token dicts for given cutomer Id
        """
        url = "{}/{}/tokens".format(self.base_url, customer_id)
        return self.get_url(url, data, **kwargs)

    def delete(self, customer_id, token_id, data={}, **kwargs):
        """"
        Delete Given Token For a Customer

        Args:
            customer_id : Customer Id for which tokens have to be deleted
            token_id    : Id for which TOken object has to be deleted
        Returns:
            Dict for deleted token
        """
        url = "{}/{}/tokens/{}".format(self.base_url, customer_id, token_id)
        return self.delete_url(url, data, **kwargs)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientPlan(ClientTestCase):

    def setUp(self):
        super(TestClientPlan, self).setUp()
        self.base_url = '{}/plans'.format(self.base_url)
        self.plan_id = 'plan_8kihN0YqhnF8a7'

    @responses.activate
    def test_plan_fetch_all(self):
        result = mock_file('plan_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.plan.all(), result)

    @responses.activate
    def test_plan_fetch(self):
        result = mock_file('fake_plan')
        url = '{}/{}'.format(self.base_url, self.plan_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.plan.fetch(self.plan_id), result)

    @responses.activate
    def test_plan_create(self):
        init = mock_file('init_plan')
        result = mock_file('fake_plan')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.plan.create(init), result)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientCustomer(ClientTestCase):

    def setUp(self):
        super(TestClientCustomer, self).setUp()
        self.base_url = '{}/customers'.format(self.base_url)

    @responses.activate
    def test_tokens_all(self):
        result = mock_file('token_collection')
        url = '{}/{}/tokens'.format(self.base_url, self.customer_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.token.all(self.customer_id),
                         result)

    @responses.activate
    def test_token_fetch(self):
        result = mock_file('token_collection')
        url = '{}/{}/tokens/{}'.format(self.base_url,
                                       self.customer_id,
                                       self.token_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(
            self.client.token.fetch(self.customer_id, self.token_id),
            result)

    @responses.activate
    def test_token_delete(self):
        url = '{}/{}/tokens/{}'.format(self.base_url,
                                       self.customer_id,
                                       self.token_id)
        responses.add(responses.DELETE,
                      url,
                      status=200,
                      body=json.dumps({'deleted': True}),
                      match_querystring=True)
        self.assertEqual(
            self.client.token.delete(self.customer_id, self.token_id),
            {'deleted': True})



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientSubscription(ClientTestCase):

    def setUp(self):
        super(TestClientSubscription, self).setUp()
        self.base_url = '{}/subscriptions'.format(self.base_url)
        self.subscription_id = 'sub_8RlLljfA4AnDVx'

    @responses.activate
    def test_subscription_fetch_all(self):
        result = mock_file('subscription_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.subscription.all(), result)

    @responses.activate
    def test_subscription_fetch(self):
        result = mock_file('fake_subscription')
        url = '{}/{}'.format(self.base_url, 'fake_subscription_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(
            self.client.subscription.fetch('fake_subscription_id'),
            result)

    @responses.activate
    def test_subscription_create(self):
        init = mock_file('init_subscription')
        result = mock_file('fake_subscription')
        url = self.base_url
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.subscription.create(init), result)

    @responses.activate
    def test_subscription_cancel(self):
        result = mock_file('fake_subscription_cancelled')
        url = '{}/{}/cancel'.format(self.base_url, self.subscription_id)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        response = json.loads(
            self.client.subscription.cancel(self.subscription_id))
        self.assertEqual(response['id'], self.subscription_id)
        self.assertEqual(response['entity'], 'subscription')
        self.assertEqual(response['status'], 'cancelled')

    @responses.activate
    def test_subscription_cancel_scheduled_changes(self):
        result = mock_file('fake_subscription_resumed')
        url = '{}/{}/cancel_scheduled_changes'.format(self.base_url, self.subscription_id)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        response = json.loads(
            self.client.subscription.cancel_scheduled_changes(self.subscription_id))
        self.assertEqual(response['id'], self.subscription_id)
        self.assertEqual(response['entity'], 'subscription')   

    @responses.activate
    def test_subscription_create_addon(self):
        result = mock_file('fake_subscription_addon')
        url = '{}/{}/addons'.format(self.base_url, self.subscription_id)
        responses.add(responses.POST,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)
        response = json.loads(
            self.client.subscription.createAddon(
                self.subscription_id,
                {'item': {'name': 'Extra Chair', 'amount': 30000,
                          'currency': 'INR'}, 'quantity': 2}))
        self.assertEqual(response['subscription_id'], self.subscription_id)
        self.assertEqual(response['entity'], 'addon')
        self.assertEqual(response['item']['name'], 'Extra Chair')
        self.assertEqual(response['item']['amount'], 30000)

    @responses.activate
    def test_subscription_edit(self):
        param = {
                 "quantity":2,
                 "schedule_change_at":"cycle_end",
               }

        result = mock_file('fake_subscription')
        url = '{}/{}'.format(self.base_url, 'subscription_id')
        responses.add(responses.PATCH, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.subscription.edit('subscription_id', param), result)     

    @responses.activate
    def test_subscription_pending_update(self):
        result = mock_file('fake_subscription')
        url = '{}/{}/retrieve_scheduled_changes'.format(self.base_url, 'fake_subscription_id')
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(
            self.client.subscription.pending_update('fake_subscription_id'),
            result)

    @responses.activate
    def test_subscription_pause(self):
        result = mock_file('fake_subscription_paused')
        url = '{}/{}/pause'.format(self.base_url, self.subscription_id)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        response = json.loads(
            self.client.subscription.pause(self.subscription_id))
        self.assertEqual(response['id'], self.subscription_id)
        self.assertEqual(response['entity'], 'subscription')
        self.assertEqual(response['status'], 'paused')  

    @responses.activate
    def test_subscription_resume(self):
        result = mock_file('fake_subscription_resumed')
        url = '{}/{}/resume'.format(self.base_url, self.subscription_id)
        responses.add(responses.POST, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        response = json.loads(
            self.client.subscription.resume(self.subscription_id))
        self.assertEqual(response['id'], self.subscription_id)
        self.assertEqual(response['entity'], 'subscription')
        self.assertEqual(response['status'], 'active')

    @responses.activate
    def test_subscription_delete_offer(self):
        result = mock_file('fake_subscription')
        url = '{}/{}/{}'.format(self.base_url, 'sub_8kip7ybbcOyc9J','offer_IjA06IHSz33cw2')
        responses.add(responses.DELETE, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        response = json.loads(
            self.client.subscription.delete_offer('sub_8kip7ybbcOyc9J','offer_IjA06IHSz33cw2'))
        self.assertEqual(response['id'], 'sub_8kip7ybbcOyc9J')
        self.assertEqual(response['entity'], 'subscription')               


from .base import Resource
from ..constants.url import URL


class Subscription(Resource):
    def __init__(self, client=None):
        super(Subscription, self).__init__(client)
        self.base_url = URL.SUBSCRIPTION_URL

    def all(self, data={}, **kwargs):
        """"
        Fetch all Subscription entities

        Returns:
            Dictionary of Subscription data
        """
        return super(Subscription, self).all(data, **kwargs)

    def fetch(self, subscription_id, data={}, **kwargs):
        """"
        Fetch Subscription for given Id

        Args:
            subscription_id : Id for which subscription object is retrieved

        Returns:
            Subscription dict for given subscription Id
        """
        return super(Subscription, self).fetch(subscription_id, data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create Subscription from given dict

        Args:
            data : Dictionary using which Subscription has to be created

        Returns:
            Subscription Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def cancel(self, subscription_id, data={}, **kwargs):
        """
        Cancel subscription given by subscription_id

        Args:
            subscription_id : Id for which subscription has to be cancelled

        Returns:
            Subscription Dict for given subscription id
        """
        url = "{}/{}/cancel".format(self.base_url, subscription_id)
        return self.post_url(url, data, **kwargs)

    def cancel_scheduled_changes(self, subscription_id, data={}, **kwargs):
        """
        Cancel a update

        Args:
            subscription_id : Id for which subscription has to be cancelled

        Returns:
            Subscription Dict for given subscription id
        """
        url = "{}/{}/cancel_scheduled_changes".format(self.base_url, subscription_id)
        return self.post_url(url, data, **kwargs)    

    def createAddon(self, subscription_id, data={}, **kwargs):
        """
        Create addon for given subscription

        Args:
            subscription_id : Id for which addon has to be created

        Return:
            Subscription dict for given subscription id
        """
        url = "{}/{}/addons".format(self.base_url, subscription_id)
        return self.post_url(url, data, **kwargs) 

    def edit(self, subscription_id, data={}, **kwargs):
        """"
         Update particular subscription

        Args:
            subscription_id : Id for which subscription has to be edited         
        Returns:
            Subscription dict for given subscription id
        """
        url = '{}/{}'.format(self.base_url, subscription_id)   
        return self.patch_url(url, data, **kwargs) 

    def pending_update(self, subscription_id, **kwargs):
        """"
        Fetch Subscription for given Id

        Args:
            subscription_id : Id for which subscription object is retrieved

        Returns:
            Subscription dict for given subscription Id
        """
        url = '{}/{}/retrieve_scheduled_changes'.format(self.base_url, subscription_id)   
        return self.get_url(url, {}, **kwargs)    

    def pause(self, subscription_id, data={}, **kwargs):
        """
        Cancel subscription given by subscription_id

        Args:
            subscription_id : Id for which subscription has to be paused

        Returns:
            Subscription Dict for given subscription id
        """
        url = "{}/{}/pause".format(self.base_url, subscription_id)
        return self.post_url(url, data, **kwargs)

    def resume(self, subscription_id, data={}, **kwargs):
        """
        Cancel subscription given by subscription_id

        Args:
            subscription_id : Id for which subscription has to be resume

        Returns:
            Subscription Dict for given subscription id
        """
        url = "{}/{}/resume".format(self.base_url, subscription_id)
        return self.post_url(url, data, **kwargs)

    def delete_offer(self, subscription_id, offer_id, data={}, **kwargs):
        """
        Delete offer linked to a subscription

        Args:
            subscription_id : The id of the subscription to offer need to be deleted
            offer_id : The id of the offer linked to subscription

        Returns:
            Subscription Dict for given subscription id
        """
        url = "{}/{}/{}".format(self.base_url, subscription_id, offer_id)
        return self.delete_url(url, data, **kwargs)                 



from .base import Resource
from ..constants.url import URL
import warnings


class Transfer(Resource):
    def __init__(self, client=None):
        super(Transfer, self).__init__(client)
        self.base_url = URL.TRANSFER_URL

    def fetch_all(self, data={}, **kwargs):  # pragma: no cover
        warnings.warn("Will be Deprecated in next release, use all",
                      DeprecationWarning)
        return self.all(data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch all Transfer entities

        Returns:
            Dictionary of Transfer data
        """
        if 'payment_id' in data:
            url = "/payments/{}/transfers".format(data['payment_id'])

            del data['payment_id']
            return self.get_url(url, data, **kwargs)

        return super(Transfer, self).all(data, **kwargs)

    def fetch(self, transfer_id, data={}, **kwargs):
        """"
        Fetch Transfer for given Id

        Args:
            transfer_id : Id for which transfer object has to be retrieved

        Returns:
            Transfer dict for given transfer Id
        """
        return super(Transfer, self).fetch(transfer_id, data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create Transfer from given dict

        Args:

        Returns:
            Transfer Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def edit(self, transfer_id, data={}, **kwargs):
        """"
        Edit Transfer from given id

        Args:
            transfer_id : Id for which transfer object has to be edited

        Returns:
            Transfer Dict which was edited
        """
        url = "{}/{}".format(self.base_url, transfer_id)
        return self.patch_url(url, data, **kwargs)

    def reverse(self, transfer_id, data={}, **kwargs):
        """"
        Reverse Transfer from given id

        Args:
            transfer_id : Id for which transfer object has to be reversed

        Returns:
            Transfer Dict which was reversed
        """
        url = "{}/{}/reversals".format(self.base_url, transfer_id)
        return self.post_url(url, data, **kwargs)

    def reversals(self, transfer_id, data={}, **kwargs):
        """"
        Get all Reversal Transfer from given id

        Args:
            transfer_id :
                Id for which reversal transfer object has to be fetched

        Returns:
            Transfer Dict
        """
        url = "{}/{}/reversals".format(self.base_url, transfer_id)
        return self.get_url(url, data, **kwargs)



from setuptools import setup

with open('README.md') as readme:
    readme_content = readme.read()

setup(
    name="thawani",
    version="1.1.2",
    description="Thawani Python Client",
    long_description=readme_content,
    long_description_content_type='text/markdown',
    url="https://github.com/muradlansa/thawani-python",
    author="Murad Lansa",
    author_email="mail@muradlansa.com",
    license="MIT",
    install_requires=["requests"],
    include_package_data=True,
    package_dir={'thawani': 'thawani', 'thawani.resources': 'thawani/resources'},
    packages=['thawani', 'thawani.resources'],
    keywords='Thawani Payment gateway Oman',

)



import responses
import json

from .helpers import mock_file, ClientTestCase


class TestClientAddon(ClientTestCase):

    def setUp(self):
        super(TestClientAddon, self).setUp()
        self.base_url = '{}/addons'.format(self.base_url)
        self.addon_id = 'ao_8sg8LU73Y3ieav'

    @responses.activate
    def test_addon_fetch(self):
        result = mock_file('fake_addon')
        url = '{}/{}'.format(self.base_url, self.addon_id)
        responses.add(responses.GET, url, status=200, body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.addon.fetch(self.addon_id), result)

    @responses.activate
    def test_addon_delete(self):
        result = []
        url = '{}/{}'.format(self.base_url, self.addon_id)
        responses.add(responses.DELETE,
                      url,
                      status=200,
                      body=json.dumps(result),
                      match_querystring=True)
        self.assertEqual(self.client.addon.delete(self.addon_id), result)

    
    @responses.activate
    def test_addon_fetch_all(self):
        result = mock_file('addon_collection')
        url = self.base_url
        responses.add(responses.GET, url, status=200,
                      body=json.dumps(result), match_querystring=True)
        self.assertEqual(self.client.addon.all(), result)       



from .base import Resource
from ..constants.url import URL
import warnings


class Invoice(Resource):
    def __init__(self, client=None):
        super(Invoice, self).__init__(client)
        self.base_url = URL.INVOICE_URL

    def fetch_all(self, data={}, **kwargs):  # pragma: no cover
        warnings.warn("Will be Deprecated in next release", DeprecationWarning)
        return self.all(data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch all Invoice entities

        Returns:
            Dictionary of Invoice data
        """
        return super(Invoice, self).all(data, **kwargs)

    def fetch(self, invoice_id, data={}, **kwargs):
        """"
        Fetch Invoice for given Id

        Args:
            invoice_id : Id for which invoice object has to be retrieved

        Returns:
            Invoice dict for given invoice Id
        """
        return super(Invoice, self).fetch(invoice_id, data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create Invoice from given dict

        Args:
            data : Dictionary having keys using which invoice have to be created

        Returns:
            Invoice Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def notify_by(self, invoice_id, medium, **kwargs):
        """"
        Send/Resend notifications to customer via email/sms

        Args:
            invoice_id : Id for trigger notify
            medium : Medium for triggering notification via email or sms

        Returns:
            {"success": true}
        """
        url = "{}/{}/notify_by/{}".format(self.base_url, invoice_id, medium)
        return self.post_url(url, {}, **kwargs)

    def cancel(self, invoice_id, **kwargs):
        """"
        Cancel an unpaid Invoice with given ID via API
        It can only be called on an invoice that is not in the paid state.

        Args:
            invoice_id : Id for cancel the invoice
        Returns:
            The response for the API will be the invoice entity, similar to create/update API response, with status attribute's value as cancelled
        """
        url = "{}/{}/cancel".format(self.base_url, invoice_id)
        return self.post_url(url, {}, **kwargs)

    def delete(self, invoice_id, **kwargs):
        """"
        Delete an invoice
        You can delete an invoice which is in the draft state.

        Args:
            invoice_id : Id for delete the invoice
        Returns:
            The response is always be an empty array like this - []
        """
        url = "{}/{}".format(self.base_url, invoice_id)
        return self.delete_url(url, {}, **kwargs)

    def issue(self, invoice_id, **kwargs):
        """"
        Issues an invoice in draft state

        Args:
            invoice_id : Id for delete the invoice
        Returns:
            Its response is the invoice entity, similar to create/update API response. Its status now would be issued.
        """
        url = "{}/{}/issue".format(self.base_url, invoice_id)
        return self.post_url(url, {}, **kwargs)

    def edit(self, invoice_id, data={}, **kwargs):
        """"
        Update an invoice
        In draft state all the attributes are allowed.

        Args:
            invoice_id : Id for delete the invoice
            data : Dictionary having keys using which invoice have to be updated
        Returns:
            Its response is the invoice entity, similar to create/update API response. Its status now would be issued.
            Refer https://thawani.com/docs/invoices/api/#entity-structure
        """
        url = "{}/{}".format(self.base_url, invoice_id)
        return self.patch_url(url, data, **kwargs)



from .base import Resource
from ..constants.url import URL
import warnings


class PaymentLink(Resource):
    def __init__(self, client=None):
        super(PaymentLink, self).__init__(client)
        self.base_url = URL.PAYMENT_LINK_URL

    def fetch_all(self, data={}, **kwargs):  # pragma: no cover
        warnings.warn("Will be Deprecated in next release", DeprecationWarning)
        return self.all(data, **kwargs)

    def all(self, data={}, **kwargs):
        """"
        Fetch all Payment link entities

        Returns:
            Dictionary of Payment link data
        """
        return super(PaymentLink, self).all(data, **kwargs)

    def fetch(self, payment_link_id, data={}, **kwargs):
        """"
        Fetch Payment link for given Id

        Args:
            payment_link_id : Id for which Payment link object has to be retrieved

        Returns:
            Payment link dict for given payment_link_id Id
        """
        return super(PaymentLink, self).fetch(payment_link_id, data, **kwargs)

    def create(self, data={}, **kwargs):
        """"
        Create Payment link from given dict

        Args:
            data : Dictionary having keys using which Payment link have to be created

        Returns:
            Payment link Dict which was created
        """
        url = self.base_url
        return self.post_url(url, data, **kwargs)

    def cancel(self, payment_link_id, **kwargs):
        """"
        Cancel an unpaid Payment link with given ID via API
        It can only be called on an Payment link that is not in the paid state.

        Args:
            payment_link_id : Id for cancel the Payment link
        Returns:
            The response for the API will be the Payment link entity, similar to create/update API response, with status attribute's value as cancelled
        """
        url = "{}/{}/cancel".format(self.base_url, payment_link_id)
        return self.post_url(url, {}, **kwargs)
   
    def edit(self, payment_link_id, data={}, **kwargs):
        """"
        Edit the Payment link
        Args:
            data : Dictionary having keys using which order have to be edited
                reference_id : Adds a unique reference number to an existing link.

                expire_by : Timestamp, in Unix format, when the payment links should expire.

                notes : key value pair as notes
            
            Returns:
            Payment Link Dict which was edited
        """
        url = '{}/{}'.format(self.base_url, payment_link_id)
        return self.patch_url(url, data, **kwargs)

    def notifyBy(self, payment_link_id, medium, **kwargs):
        """"
        Send notification

        Args:
            payment_link_id : Unique identifier of the Payment Link that should be resent.
            
            medium : sms/email
        """
        url = "{}/{}/notify_by/{}".format(self.base_url, payment_link_id, medium)
        return self.post_url(url, {}, **kwargs) 



import thawani
import os
import unittest


def mock_file(filename):
    if not filename:
        return ''
    file_dir = os.path.dirname(__file__)
    file_path = "{}/mocks/{}.json".format(file_dir, filename)
    with open(file_path) as f:
        mock_file_data = f.read()
    return mock_file_data


class ClientTestCase(unittest.TestCase):
    def setUp(self):
        self.base_url = 'https://api.thawani.com/v1'
        self.secondary_url = 'https://test-api.thawani.com/v1'
        self.payment_id = 'fake_payment_id'
        self.refund_id = 'fake_refund_id'
        self.card_id = 'fake_card_id'
        self.customer_id = 'fake_customer_id'
        self.token_id = 'fake_token_id'
        self.addon_id = 'fake_addon_id'
        self.subscription_id = 'fake_subscription_id'
        self.plan_id = 'fake_plan_id'
        self.settlement_id = 'fake_settlement_id'
        self.client = thawani.Client(auth=('key_id', 'key_secret'))
        self.secondary_client = thawani.Client(auth=('key_id', 'key_secret'),
                                                base_url=self.secondary_url)



from .base import Resource
from ..constants.url import URL


class Card(Resource):
    def __init__(self, client=None):
        super(Card, self).__init__(client)
        self.base_url = URL.CARD_URL

    def fetch(self, card_id, data={}, **kwargs):
        """"
        Fetch Card for given Id

        Args:
            card_id : Id for which card object has to be retrieved

        Returns:
            Card dict for given card Id
        """
        return super(Card, self).fetch(card_id, data, **kwargs)
